function [lambdas selectedFeatures] = modelPursuit(obsPoints, negPoints, featureBank,numWeakClassifiers, folderName)

% =================================================================
% Code was originally designed for comparison adaboost and other 
%   models, thus not in the most compact form. 
% To make it more clear, unrelated code is deleted.
% 
% <obsPoints>: a matrix where each row denotes binary responses
%   of a positive example to features in the feature bank.
% <negPoints>: similar to <obsPoints), while each row is binary
%   responses of a negative example.
% <featureBank>: set of features to select from.
% <numWeakClassifiers>: number of weak classifiers to choose.
% <folderName>: output folder of results.
%
% Adaboost implementation follows the version formulated by (Viola, Jones 2001).
% Cascading is NOT performed.
%
% =================================================================

folder = folderName;
if(~exist(folder,'dir'))
    mkdir(folder);
end


dataPoints = [obsPoints;negPoints];

selectedFeatures = [];
trueLabel = [ones(size(obsPoints,1),1);zeros(size(negPoints,1),1)];
lambdas = zeros(numWeakClassifiers,1);
weights = [ones(size(obsPoints,1),1)/size(obsPoints,1);ones(size(negPoints,1),1)/size(negPoints,1)]/2;

for step=1:numWeakClassifiers
    % ---------------------------------
    % select feature / weak classifier
    % ---------------------------------
    newSelectedFeature = struct('name','na','params',[],'pol',0,'benifit',0,'binary',1,'EobsH',-1); % pol means polar of the binary feature

    % Begin feature selection
    featureCount = 1;
    benefitVec = []; % benefit of each feature
                     % for adaboost, we use: benefit = 1-errorRate
    for i = 1:size(featureBank,1) % iterate over all feature types (only Gabor here)
                                  % features are binary classifiers built on Gabor basis response 
        for iParams = 1:size(featureBank(i).paramRange,1) % iterate all parameters (location, scale)
            params = featureBank(i).paramRange(iParams,:);
            binaryResponseVec = dataPoints(:,params(1));
            % weighted sum of binary predictions on positive images (correct rate on positives)
            Eobs = sum(binaryResponseVec(1:size(obsPoints,1)).*weights(1:size(obsPoints,1)) );
            % weighted sum of binary predictions on negative images (error rate on negatives)
            Esyn = sum( binaryResponseVec( (size(obsPoints,1)+1):end).*weights( (size(obsPoints,1)+1) :end) );
            % calculate error rate
            errorRate = Esyn + (sum(weights(1:size(obsPoints,1))) - Eobs);
            if errorRate < 0.5
                pol = 1; % polar of classifier
                benefit = 1 - errorRate;
            else
                pol = 0;
                benefit = errorRate;
            end
            benefitVec(featureCount) = benefit;
            polVec(featureCount) = pol;
            featureCount = featureCount + 1;
        end
    end
   
    % Select the feature with largest benefit
    %   If there are multiple largest, break tie arbitrarily
    [maxValues maxInds] = max(benefitVec);
    tmpIndex = floor((1+length(maxValues))/2);
    newFeatureIndex = maxInds(tmpIndex);
    disp(['feature selected: No. ' num2str(newFeatureIndex)]);
    fCount = 1;
    % Search for the feature type and parameters given index of this feature <newFeatureIndex>   
    %  (To search for it is a little clumpsy, but it is not so time consuming compared to other parts of program) 
    for i = 1:size(featureBank,1)
        for iParams = 1:size(featureBank(i).paramRange,1)
            if(fCount == newFeatureIndex)
                params = featureBank(i).paramRange(iParams,:);
                newSelectedFeature.name = featureBank(i).name;
                newSelectedFeature.params = params;
                newSelectedFeature.pol = polVec(fCount);
                newSelectedFeature.benefit = benefitVec(fCount);
                disp(['select basis with errorRate = ' num2str(1-newSelectedFeature.benefit)]);
                break;
            end
            fCount = fCount + 1;
        end
    end
    % Add to <selectedFeatures>
    selectedFeatures = [selectedFeatures;newSelectedFeature];

    %-----------------------------------
    % Calculate weight of new weak classifier
    %-----------------------------------
    % beta is (error)/(1-error)
    % Note: we allow error to be larger than 0.5, thus allowing lambda
    % to be negative
    if selectedFeatures(end).pol == 1
        errorNewFeature = 1 - selectedFeatures(end).benefit;
    else
        errorNewFeature = selectedFeatures(end).benefit;
    end
    beta = errorNewFeature / (1-errorNewFeature);
    newLambda = -log(beta+1e-6);
    lambdas(step) = newLambda;
    % update the weights
    binaryResponseVec = calcFeatureResponse([obsPoints;negPoints],selectedFeatures(end).name,selectedFeatures(end).params);
    weights = weights .* (beta .^ (binaryResponseVec==trueLabel));
    weights = weights/sum(weights(:));

    %-----------------------------------
    % calculate error rate of current strong classifier
    %-----------------------------------
    scoreVec = zeros(size([obsPoints;negPoints],1),1);
    for k = 1:step
        binaryResponseVec = calcFeatureResponse([obsPoints;negPoints],selectedFeatures(k).name,selectedFeatures(k).params);
        scoreVec = scoreVec + binaryResponseVec * lambdas(k);
    end
    % For display purpose
    threshold = sum(lambdas(1:step))/2;
    errorRateStrongClassifier(step) = sum((scoreVec>=threshold) ~= trueLabel)/size([obsPoints;negPoints],1);
    gt = trueLabel; gt(gt==0) = -1;
    auc(k) = calc_auc(scoreVec,gt);
    disp(['after step ' num2str(k) ', strong classifier errorRate=' num2str(errorRateStrongClassifier(step))]);
    save([folder '/' folderName '.mat'],'negPoints','obsPoints','lambdas','selectedFeatures','errorRateStrongClassifier','auc');
end

h=figure;plot(errorRateStrongClassifier);
saveas(h,[folder '/trainerr.eps']);
xlabel('# selected basis');
title('train error');
close;
h=figure;plot(auc);
saveas(h,[folderName '/trainauc.eps']);
xlabel('# selected basis');
title('train auc');
close;